package javax.swing.plaf;

import java.awt.Font;
import javax.swing.border.TitledBorder;
import javax.swing.Icon;
import javax.swing.border.MatteBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.BevelBorder;
import javax.swing.border.LineBorder;
import javax.swing.border.EmptyBorder;
import java.beans.ConstructorProperties;
import javax.swing.border.CompoundBorder;
import java.awt.Insets;
import java.awt.Graphics;
import java.awt.Component;
import java.awt.Color;
import java.io.Serializable;
import javax.swing.border.Border;

public class BorderUIResource implements Border, UIResource, Serializable
{
    static Border etched;
    static Border loweredBevel;
    static Border raisedBevel;
    static Border blackLine;
    private Border delegate;
    
    public static Border getEtchedBorderUIResource() {
        if (BorderUIResource.etched == null) {
            BorderUIResource.etched = new BorderUIResource.EtchedBorderUIResource();
        }
        return BorderUIResource.etched;
    }
    
    public static Border getLoweredBevelBorderUIResource() {
        if (BorderUIResource.loweredBevel == null) {
            BorderUIResource.loweredBevel = new BorderUIResource.BevelBorderUIResource(1);
        }
        return BorderUIResource.loweredBevel;
    }
    
    public static Border getRaisedBevelBorderUIResource() {
        if (BorderUIResource.raisedBevel == null) {
            BorderUIResource.raisedBevel = new BorderUIResource.BevelBorderUIResource(0);
        }
        return BorderUIResource.raisedBevel;
    }
    
    public static Border getBlackLineBorderUIResource() {
        if (BorderUIResource.blackLine == null) {
            BorderUIResource.blackLine = new BorderUIResource.LineBorderUIResource(Color.black);
        }
        return BorderUIResource.blackLine;
    }
    
    public BorderUIResource(Border delegate) {
        if (delegate == null) {
            throw new IllegalArgumentException("null border delegate argument");
        }
        this.delegate = delegate;
        return;
    }
    
    @Override
    public void paintBorder(Component component, Graphics graphics, int n, int n2, int n3, int n4) {
        this.delegate.paintBorder(component, graphics, n, n2, n3, n4);
    }
    
    @Override
    public Insets getBorderInsets(Component component) {
        return this.delegate.getBorderInsets(component);
    }
    
    @Override
    public boolean isBorderOpaque() {
        return this.delegate.isBorderOpaque();
    }
    
    public static class CompoundBorderUIResource extends CompoundBorder implements UIResource
    {
        @ConstructorProperties({ "outsideBorder", "insideBorder" })
        public CompoundBorderUIResource(Border outsideBorder, Border insideBorder) {
            super(outsideBorder, insideBorder);
        }
    }
    
    public static class EmptyBorderUIResource extends EmptyBorder implements UIResource
    {
        public EmptyBorderUIResource(int top, int left, int bottom, int right) {
            super(top, left, bottom, right);
        }
        
        @ConstructorProperties({ "borderInsets" })
        public EmptyBorderUIResource(Insets borderInsets) {
            super(borderInsets);
        }
    }
    
    public static class LineBorderUIResource extends LineBorder implements UIResource
    {
        public LineBorderUIResource(Color color) {
            super(color);
        }
        
        @ConstructorProperties({ "lineColor", "thickness" })
        public LineBorderUIResource(Color color, int thickness) {
            super(color, thickness);
        }
    }
    
    public static class BevelBorderUIResource extends BevelBorder implements UIResource
    {
        public BevelBorderUIResource(int bevelType) {
            super(bevelType);
        }
        
        public BevelBorderUIResource(int bevelType, Color highlight, Color shadow) {
            super(bevelType, highlight, shadow);
        }
        
        @ConstructorProperties({ "bevelType", "highlightOuterColor", "highlightInnerColor", "shadowOuterColor", "shadowInnerColor" })
        public BevelBorderUIResource(int bevelType, Color highlightOuterColor, Color highlightInnerColor, Color shadowOuterColor, Color shadowInnerColor) {
            super(bevelType, highlightOuterColor, highlightInnerColor, shadowOuterColor, shadowInnerColor);
        }
    }
    
    public static class EtchedBorderUIResource extends EtchedBorder implements UIResource
    {
        public EtchedBorderUIResource() {
        }
        
        public EtchedBorderUIResource(int etchType) {
            super(etchType);
        }
        
        public EtchedBorderUIResource(Color highlight, Color shadow) {
            super(highlight, shadow);
        }
        
        @ConstructorProperties({ "etchType", "highlightColor", "shadowColor" })
        public EtchedBorderUIResource(int etchType, Color highlight, Color shadow) {
            super(etchType, highlight, shadow);
        }
    }
    
    public static class MatteBorderUIResource extends MatteBorder implements UIResource
    {
        public MatteBorderUIResource(int top, int left, int bottom, int right, Color matteColor) {
            super(top, left, bottom, right, matteColor);
        }
        
        public MatteBorderUIResource(int top, int left, int bottom, int right, Icon tileIcon) {
            super(top, left, bottom, right, tileIcon);
        }
        
        public MatteBorderUIResource(Icon tileIcon) {
            super(tileIcon);
        }
    }
    
    public static class TitledBorderUIResource extends TitledBorder implements UIResource
    {
        public TitledBorderUIResource(String title) {
            super(title);
        }
        
        public TitledBorderUIResource(Border border) {
            super(border);
        }
        
        public TitledBorderUIResource(Border border, String title) {
            super(border, title);
        }
        
        public TitledBorderUIResource(Border border, String title, int titleJustification, int titlePosition) {
            super(border, title, titleJustification, titlePosition);
        }
        
        public TitledBorderUIResource(Border border, String title, int titleJustification, int titlePosition, Font titleFont) {
            super(border, title, titleJustification, titlePosition, titleFont);
        }
        
        @ConstructorProperties({ "border", "title", "titleJustification", "titlePosition", "titleFont", "titleColor" })
        public TitledBorderUIResource(Border border, String title, int titleJustification, int titlePosition, Font titleFont, Color titleColor) {
            super(border, title, titleJustification, titlePosition, titleFont, titleColor);
        }
    }
}
